@@ -35,6 +35,7 @@ EXHAUST_DNS = True
 class TickTimer:
 	def __init__ (self):
 		self.onces = []
+		self._canceled = {}
 		self._call_id = 0
 		self._lock = threading.Lock ()
 
@@ -46,16 +47,15 @@ class TickTimer:
 		with self._lock:
 			self._call_id += 1
 			call_id = self._call_id
+			last_at = self.onces and self.onces [-1][1] or None
 			self.onces.append ((call_id, at, func, args))
-			self.onces = sorted (self.onces, key = lambda x: x [1])
+			if last_at and at < last_at:
+				self.onces.sort (key = lambda x: x [1])
 		return call_id
 
 	def cancel (self, call_id):
 		with self._lock:
-			for idx, call in enumerate (self.onces):
-				if call_id == call [0]:
-					self.onces.pop (idx)
-					break
+			self._canceled [call_id] = None
 
 	def tick (self):
 		with self._lock:
@@ -69,10 +69,13 @@ class TickTimer:
 				if at > now:
 					break
 				call_id, at, func, args = self.onces.pop (0)
+				if call_id in self._canceled:
+					self._canceled.pop (call_id)
+					continue
 				funcs.append ((func, args))
 		for func, args in funcs:
 			func (*args)
-		return now
+
 
 class Maintern:
 	def __init__ (self):
@@ -230,27 +233,10 @@ def remove_notsocks (map):
 
 	return killed
 
-PREFER_EPOLL = False
-ALLOWED_POLL_ERROR = 8
 poll_fun = asyncore.poll
-if os.name == 'posix' and '__pypy__' not in sys.builtin_module_names:
-	poll_fun = asyncore.epoll
-	PREFER_EPOLL = True
-poll_errs = 0
-
-def maybe_poll_fallback ():
-	global _logger, poll_fun, poll_errs
-
-	poll_errs += 1
-	if poll_errs > ALLOWED_POLL_ERROR:
-		if poll_fun is asyncore.poll:
-			raise SystemError ('Too many polling error')
-		_logger and _logger ('fallback to asyncore.poll', 'warn')
-		poll_fun = asyncore.poll # fallback
-		poll_errs = 0
 
 def poll_fun_wrap (timeout, map = None):
-	global _logger, poll_fun, poll_errs
+	global _logger, poll_fun
 
 	if map is None:
 		map = asyncore.socket_map
@@ -266,12 +252,10 @@ def poll_fun_wrap (timeout, map = None):
 
 	except (TypeError, OSError) as why:
 		_logger and _logger.trace ()
-		maybe_poll_fallback ()
 		remove_notsocks (map) # WSAENOTSOCK
 
 	except ValueError:
 		_logger and _logger.trace ()
-		maybe_poll_fallback ()
 		# negative file descriptor, testing all sockets
 		# or too many file descriptors in select(), divide and conquer
 		if not remove_notsocks (map):
@@ -288,21 +272,13 @@ def poll_fun_wrap (timeout, map = None):
 
 	except:
 		_logger and _logger.trace ()
-		if poll_fun is asyncore.poll:
-			raise
-		_logger and _logger ('fallback to asyncore.poll', 'warn')
-		poll_fun = asyncore.poll # fallback
+		raise
 
-	else:
-		if poll_errs:
-			poll_errs = 0 # reset error count
-			if PREFER_EPOLL:
-				_logger and _logger ('reinstate asyncore.epoll', 'info')
-				poll_fun = asyncore.epoll
 
 def lifetime_loop (timeout = 30.0, count = 0):
 	global _last_maintern
 	global _maintern_interval
+	global tick_timer
 
 	loop = 0
 	map = asyncore.socket_map
@@ -320,6 +296,8 @@ def lifetime_loop (timeout = 30.0, count = 0):
 
 def graceful_shutdown_loop ():
 	global _shutdown_phase
+	global tick_timer
+
 	timestamp = time.time()
 	timeout = 1.0
 	map = asyncore.socket_map
@@ -339,6 +317,7 @@ def graceful_shutdown_loop ():
 
 		if veto and time_in_this_phase < _shutdown_timeout:
 			poll_fun_wrap (timeout, map)
+			tick_timer.tick ()
 		else:
 			_shutdown_phase += 1
 			timestamp = time.time()
