@@ -12,9 +12,3 @@ class PushCanceled (H3Event):
 @dataclass
 class MaxPushIdReceived (H3Event):
     push_id: int
-
-@dataclass
-class DuplicatePushReceived (H3Event):
-    stream_id: int
-    push_id: int
-    stream_ended: bool = True
